#ifndef Assembler_hpp
#define Assembler_hpp

#include "Operand.hpp"

namespace SoftWire
{
	class Synthesizer;
	class Instruction;
	class Scanner;
	class Parser;
	class Linker;
	class Loader;
	class Error;
	class InstructionSet;

	class Assembler
	{
	public:
		Assembler();
		Assembler(const char *fileName);

		~Assembler();

		const void (*callable(const char *entryPoint = 0))();

		static void defineExternal(void *pointer, const char *name);
		static void defineSymbol(int value, const char *name);

		const char *getErrors() const;
		const char *getListing() const;
		void clearListing() const;

		void label(const char *label);

		#include "Intrinsics.hpp"

	private:
		char *entryPoint;

		Scanner *scanner;
		Parser *parser;
		Synthesizer *synthesizer;
		InstructionSet *instructionSet;
		Linker *linker;
		Loader *loader;

		char *errors;

		void assembleFile();
		void assembleLine();

		void x86(int instructionID,
		         const Operand &firstOperand = VOID,
		         const Operand &secondOperand = VOID,
		         const Operand &thirdOperand = VOID);   // Assemble run-time intrinsic

		void handleError(const char *error);
	};

	#define ASM_EXPORT(x) Assembler::defineExternal((void*)&x, #x);
	#define ASM_DEFINE(x) Assembler::defineSymbol(x, #x);
}

#endif   // Assembler_hpp
